@@ -7,7 +7,8 @@ except ImportError:
     from urllib.request import urlopen  # py3
 from traitlets import (
     observe,
-    Bool, Bytes, Dict, Instance, Int, List, TraitError, Unicode, validate
+    Bool, Bytes, Dict, Instance, Int, List, TraitError, Unicode, validate,
+    Undefined
 )
 from ipywidgets import DOMWidget, Image, Video, Audio, register, widget_serialization
 from ipython_genutils.py3compat import string_types
@@ -348,6 +349,13 @@ class ImageRecorder(Recorder):
     _width = Unicode().tag(sync=True)
     _height = Unicode().tag(sync=True)
 
+    def __init__(self, format='png', filename=Undefined, recording=False, autosave=False, **kwargs):
+        super(ImageRecorder, self).__init__(
+            format=format, filename=filename, recording=recording, autosave=autosave, **kwargs)
+        if 'image' not in kwargs:
+            # Set up initial observer on child:
+            self.image.observe(self._check_autosave, 'value')
+
     @traitlets.default('image')
     def _default_image(self):
         return Image(width=self._width, height=self._height, format=self.format)
@@ -397,11 +405,22 @@ class ImageRecorder(Recorder):
 class VideoRecorder(Recorder):
     """Creates a recorder which allows to record a MediaStream widget, play the
     record in the Notebook, and download it or turn it into a Video widget.
+
+    For help on supported values for the "codecs" attribute, see
+    https://stackoverflow.com/questions/41739837/all-mime-types-supported-by-mediarecorder-in-firefox-and-chrome
     """
     _model_name = Unicode('VideoRecorderModel').tag(sync=True)
     _view_name = Unicode('VideoRecorderView').tag(sync=True)
 
     video = Instance(Video).tag(sync=True, **widget_serialization)
+    codecs = Unicode('', help='Optional codecs for the recording, e.g. "vp8" or "vp9, opus".').tag(sync=True)
+
+    def __init__(self, format='webm', filename=Undefined, recording=False, autosave=False, **kwargs):
+        super(VideoRecorder, self).__init__(
+            format=format, filename=filename, recording=recording, autosave=autosave, **kwargs)
+        if 'video' not in kwargs:
+            # Set up initial observer on child:
+            self.video.observe(self._check_autosave, 'value')
 
     @traitlets.default('video')
     def _default_video(self):
@@ -444,11 +463,22 @@ class VideoRecorder(Recorder):
 class AudioRecorder(Recorder):
     """Creates a recorder which allows to record the Audio of a MediaStream widget, play the
     record in the Notebook, and download it or turn it into an Audio widget.
+
+    For help on supported values for the "codecs" attribute, see
+    https://stackoverflow.com/questions/41739837/all-mime-types-supported-by-mediarecorder-in-firefox-and-chrome
     """
     _model_name = Unicode('AudioRecorderModel').tag(sync=True)
     _view_name = Unicode('AudioRecorderView').tag(sync=True)
 
     audio = Instance(Audio).tag(sync=True, **widget_serialization)
+    codecs = Unicode('', help='Optional codecs for the recording, e.g. "opus".').tag(sync=True)
+
+    def __init__(self, format='webm', filename=Undefined, recording=False, autosave=False, **kwargs):
+        super(AudioRecorder, self).__init__(
+            format=format, filename=filename, recording=recording, autosave=autosave, **kwargs)
+        if 'audio' not in kwargs:
+            # Set up initial observer on child:
+            self.audio.observe(self._check_autosave, 'value')
 
     @traitlets.default('audio')
     def _default_audio(self):
@@ -459,7 +489,7 @@ class AudioRecorder(Recorder):
         self.audio.format = self.format
 
     @observe('audio')
-    def _bind_video(self, change):
+    def _bind_audio(self, change):
         if change.old:
             change.old.unobserve(self._check_autosave, 'value')
         change.new.observe(self._check_autosave, 'value')
